@@ -140,10 +140,10 @@ class Generator:
 
         if isinstance(expression, Token):
             text = expression.text.replace(Tokenizer.ESCAPE_CODE, self.escape)
-            if expression.token_type == TokenType.IDENTIFIER:
-                text = f"{self.identifier}{text[1:-1]}{self.identifier}"
-            elif self.identify and identify:
+            if expression.token_type == TokenType.IDENTIFIER or (self.identify and identify):
                 text = f"{self.identifier}{text}{self.identifier}"
+            elif expression.token_type == TokenType.STRING:
+                return f"{self.quote}{text}{self.quote}"
             return text
 
         return getattr(self, f"{expression.key}_sql")(expression)
@@ -195,7 +195,8 @@ class Generator:
         ] if part)
 
     def from_sql(self, expression):
-        return f"{self.seg('FROM')} {self.sql(expression, 'this')}"
+        expressions = ', '.join(self.sql(e) for e in expression.args['expressions'])
+        return f"{self.seg('FROM')} {expressions}"
 
     def group_sql(self, expression):
         return self.op_expressions('GROUP BY', expression)
@@ -229,10 +230,12 @@ class Generator:
         return f"{self.seg('LIMIT')} {self.sql(expression, 'this')}"
 
     def order_sql(self, expression, flat=False):
-        sql = self.op_expressions('ORDER BY', expression, flat=flat)
-        if expression.args['desc']:
-            sql = f"{sql} DESC"
-        return sql
+        return self.op_expressions('ORDER BY', expression, flat=flat)
+
+    def ordered_sql(self, expression):
+        desc = self.sql(expression, 'desc')
+        desc = f" {desc}" if desc else ''
+        return f"{self.sql(expression, 'this')}{desc}"
 
     def select_sql(self, expression):
         hint = self.sql(expression, 'hint')
@@ -385,8 +388,20 @@ class Generator:
         distinct = 'DISTINCT ' if expression.args['distinct'] else ''
         return f"COUNT({distinct}{self.sql(expression, 'this')})"
 
+    def div_sql(self, expression):
+        return self.sql(exp.Cast(
+            this=exp.Slash(
+                this=expression.args['this'],
+                expression=expression.args['expression']
+            ),
+            to=Token(TokenType.INT, 'INT'),
+        ))
+
+    def dpipe_sql(self, expression):
+        return self.binary(expression, '||')
+
     def dot_sql(self, expression):
-        return self.binary(expression, '.')
+        return f"{self.sql(expression, 'this')}.{self.sql(expression, 'expression')}"
 
     def eq_sql(self, expression):
         return self.binary(expression, '=')
@@ -451,10 +466,7 @@ class Generator:
         )
 
     def op_expressions(self, op, expression, flat=False):
-        this_sql = self.sql(expression, 'this')
-        op_sql = self.seg(op)
         expressions_sql = self.expressions(expression, flat=flat)
         if flat:
-            this_sql = f"{this_sql} " if this_sql else ''
-            return f"{this_sql}{op} {expressions_sql}"
-        return f"{this_sql}{op_sql}{self.sep()}{expressions_sql}"
+            return f"{op} {expressions_sql}"
+        return f"{self.seg(op)}{self.sep()}{expressions_sql}"
