@@ -8,6 +8,8 @@ import sqlglot.expressions as exp
 
 os.system('')
 
+def expressions_to_map(*expressions):
+    return {expression.token_type: expression for expression in expressions}
 
 class Parser:
     def _parse_decimal(args):
@@ -93,11 +95,49 @@ class Parser:
     }
 
     COLUMN_TOKENS = {
-        TokenType.VAR,
-        TokenType.IDENTIFIER,
+        *ID_VAR_TOKENS,
         TokenType.STAR,
     }
 
+    CONJUNCTION = expressions_to_map(
+        exp.And,
+        exp.Or,
+    )
+
+    EQUALITY = expressions_to_map(
+        exp.EQ,
+        exp.NEQ,
+        exp.Is,
+    )
+
+    COMPARISON = expressions_to_map(
+        exp.GT,
+        exp.GTE,
+        exp.LT,
+        exp.LTE,
+    )
+
+    BITWISE = expressions_to_map(
+        exp.BitwiseLeftShift,
+        exp.BitwiseRightShift,
+        exp.BitwiseAnd,
+        exp.BitwiseXor,
+        exp.BitwiseOr,
+        exp.DPipe,
+    )
+
+    TERM = expressions_to_map(
+        exp.Minus,
+        exp.Plus,
+        exp.Mod,
+    )
+
+    FACTOR = expressions_to_map(
+        exp.Div,
+        exp.Slash,
+        exp.Star,
+    )
+
     def __init__(self, **opts):
         self.functions = {**self.FUNCTIONS, **(opts.get('functions') or {})}
         self.error_level = opts.get('error_level') or ErrorLevel.RAISE
@@ -124,8 +164,8 @@ class Parser:
 
         for tokens in self._chunks:
             self._index = -1
-            self._advance()
             self._tokens = tokens
+            self._advance()
             expressions.append(self._parse_statement())
 
             if self._index < len(self._tokens):
@@ -171,24 +211,8 @@ class Parser:
 
     def _advance(self):
         self._index += 1
-
-    @property
-    def _prev(self):
-        return self._safe_get(self._index - 1)
-
-    @property
-    def _curr(self):
-        return self._safe_get(self._index)
-
-    @property
-    def _next(self):
-        return self._safe_get(self._index + 1)
-
-    def _safe_get(self, index):
-        try:
-            return self._tokens[index]
-        except IndexError:
-            return None
+        self._curr = list_get(self._tokens, self._index)
+        self._prev = list_get(self._tokens, self._index - 1) if self._index > 0 else None
 
     def _parse_statement(self):
         if self._curr is None:
@@ -331,7 +355,7 @@ class Parser:
         if not self._match(TokenType.FROM):
             return None
 
-        return exp.From(this=self._parse_table())
+        return exp.From(expressions=self._parse_csv(self._parse_table))
 
     def _parse_laterals(self):
         laterals = []
@@ -371,7 +395,7 @@ class Parser:
                 this=self._parse_table(),
                 side=side,
                 kind=kind,
-                on=self._parse_expression() if self._match(TokenType.ON) else None,
+                on=self._parse_conjunction() if self._match(TokenType.ON) else None,
             ))
 
     def _parse_table(self, alias=None):
@@ -403,7 +427,7 @@ class Parser:
             this = self._parse_alias(expression)
 
         # some dialects allow not having an alias after a nested sql
-        if this.token_type != TokenType.ALIAS:
+        if this.token_type != TokenType.ALIAS and this.token_type != TokenType.TABLE :
             this = exp.Alias(this=this, alias=None)
 
         return this
@@ -444,7 +468,7 @@ class Parser:
         if not self._match(TokenType.GROUP):
             return None
 
-        return exp.Group(expressions=self._parse_csv(self._parse_primary))
+        return exp.Group(expressions=self._parse_csv(self._parse_conjunction))
 
     def _parse_having(self):
         if not self._match(TokenType.HAVING):
@@ -455,7 +479,13 @@ class Parser:
         if not self._match(TokenType.ORDER):
             return None
 
-        return exp.Order(expressions=self._parse_csv(self._parse_primary), desc=self._match(TokenType.DESC))
+        return exp.Order(expressions=self._parse_csv(self._parse_ordered))
+
+    def _parse_ordered(self):
+        return exp.Ordered(
+            this=self._parse_primary(),
+            desc=not self._match(TokenType.ASC) and self._match(TokenType.DESC),
+        )
 
     def _parse_limit(self):
         if not self._match(TokenType.LIMIT):
@@ -472,16 +502,16 @@ class Parser:
         return exp.Union(this=this, expression=self._parse_select(), distinct=distinct)
 
     def _parse_expression(self):
-        return self._parse_alias(self._parse_window(self._parse_conjunction()))
+        return self._parse_alias(self._parse_conjunction())
 
     def _parse_conjunction(self):
-        return self._parse_tokens(self._parse_equality, exp.And, exp.Or)
+        return self._parse_tokens(self._parse_equality, self.CONJUNCTION)
 
     def _parse_equality(self):
-        return self._parse_tokens(self._parse_comparison, exp.EQ, exp.NEQ, exp.Is)
+        return self._parse_tokens(self._parse_comparison, self.EQUALITY)
 
     def _parse_comparison(self):
-        return self._parse_tokens(self._parse_range, exp.GT, exp.GTE, exp.LT, exp.LTE)
+        return self._parse_tokens(self._parse_range, self.COMPARISON)
 
     def _parse_range(self):
         this = self._parse_bitwise()
@@ -511,20 +541,13 @@ class Parser:
         return this
 
     def _parse_bitwise(self):
-        return self._parse_tokens(
-            self._parse_term,
-            exp.BitwiseLeftShift,
-            exp.BitwiseRightShift,
-            exp.BitwiseAnd,
-            exp.BitwiseXor,
-            exp.BitwiseOr,
-        )
+        return self._parse_tokens(self._parse_term, self.BITWISE)
 
     def _parse_term(self):
-        return self._parse_tokens(self._parse_factor, exp.Minus, exp.Plus, exp.Mod)
+        return self._parse_tokens(self._parse_factor, self.TERM)
 
     def _parse_factor(self):
-        return self._parse_tokens(self._parse_unary, exp.Slash, exp.Star)
+        return self._parse_tokens(self._parse_unary, self.FACTOR)
 
     def _parse_unary(self):
         if self._match(TokenType.NOT):
@@ -548,21 +571,21 @@ class Parser:
         ifs = []
         default = None
 
-        expression = self._parse_expression()
+        expression = self._parse_conjunction()
 
         while self._match(TokenType.WHEN):
-            this = self._parse_expression()
+            this = self._parse_conjunction()
             self._match(TokenType.THEN)
-            then = self._parse_expression()
+            then = self._parse_conjunction()
             ifs.append(exp.If(this=this, true=then))
 
         if self._match(TokenType.ELSE):
-            default = self._parse_expression()
+            default = self._parse_conjunction()
 
         if not self._match(TokenType.END):
             self.raise_error('Expected END after CASE', self._prev)
 
-        return exp.Case(this=expression, ifs=ifs, default=default)
+        return self._parse_brackets(exp.Case(this=expression, ifs=ifs, default=default))
 
     def _parse_count(self):
         if not self._match(TokenType.L_PAREN):
@@ -574,7 +597,7 @@ class Parser:
         if not self._match(TokenType.R_PAREN):
             self.raise_error("Expected ) after COUNT")
 
-        return exp.Count(this=this, distinct=distinct)
+        return self._parse_window(exp.Count(this=this, distinct=distinct))
 
     def _parse_cast(self):
         if not self._match(TokenType.L_PAREN):
@@ -599,21 +622,64 @@ class Parser:
         if not self._match(TokenType.L_PAREN):
             return this
 
-        args = self._parse_csv(self._parse_expression)
+        args = self._parse_csv(self._parse_conjunction)
         function = self.functions.get(this.text.upper())
 
         if not self._match(TokenType.R_PAREN):
             self.raise_error('Expected )')
 
         if not callable(function):
-            return exp.Anonymous(this=this, expressions=args)
+            this = exp.Anonymous(this=this, expressions=args)
+        else:
+            this = function(args)
 
-        return function(args)
+        return self._parse_window(this)
+
+    def _parse_window(self, this):
+        if not self._match(TokenType.OVER):
+            return this
+
+        if not self._match(TokenType.L_PAREN):
+            self.raise_error('Expecting ( after OVER')
+
+        partition = None
+
+        if self._match(TokenType.PARTITION):
+            partition = self._parse_csv(self._parse_primary)
+
+        order = self._parse_order()
+
+        spec = None
+        kind = self._match(TokenType.ROWS, TokenType.RANGE)
+
+        if kind:
+            self._match(TokenType.BETWEEN)
+            start = self._parse_window_spec()
+            self._match(TokenType.AND)
+            end = self._parse_window_spec()
+
+            spec = exp.WindowSpec(
+                kind=kind,
+                start=start['value'],
+                start_side=start['side'],
+                end=end['value'],
+                end_side=end['side'],
+            )
+
+        if not self._match(TokenType.R_PAREN):
+            self.raise_error('Expecting )')
+
+        return exp.Window(this=this, partition=partition, order=order, spec=spec)
 
     def _parse_primary(self):
         if self._match(*self.PRIMARY_TOKENS):
             return self._prev
 
+        time_type = self._match(TokenType.TIMESTAMP, TokenType.DATE)
+
+        if time_type:
+            return exp.Cast(this=self._parse_primary(), to=time_type)
+
         interval = self._parse_interval()
 
         if interval:
@@ -621,7 +687,7 @@ class Parser:
 
         if self._match(TokenType.L_PAREN):
             paren = self._prev
-            this = self._parse_expression()
+            this = self._parse_conjunction()
 
             if not self._match(TokenType.R_PAREN):
                 self.raise_error('Expecting )', paren)
@@ -664,7 +730,7 @@ class Parser:
         if not self._match(TokenType.L_BRACKET):
             return this
 
-        expressions = self._parse_csv(self._parse_primary)
+        expressions = self._parse_csv(self._parse_conjunction)
 
         if isinstance(this, Token) and this.token_type == TokenType.ARRAY:
             bracket = exp.Array(expressions=expressions)
@@ -674,43 +740,12 @@ class Parser:
         if not self._match(TokenType.R_BRACKET):
             self.raise_error('Expected ]')
 
-        return self._parse_brackets(bracket)
+        return self._parse_brackets(self._parse_dot(bracket))
 
-    def _parse_window(self, this):
-        if not self._match(TokenType.OVER):
-            return this
-
-        if not self._match(TokenType.L_PAREN):
-            self.raise_error('Expecting ( after OVER')
-
-        partition = None
-
-        if self._match(TokenType.PARTITION):
-            partition = self._parse_csv(self._parse_primary)
-
-        order = self._parse_order()
-
-        spec = None
-        kind = self._match(TokenType.ROWS, TokenType.RANGE)
-
-        if kind:
-            self._match(TokenType.BETWEEN)
-            start = self._parse_window_spec()
-            self._match(TokenType.AND)
-            end = self._parse_window_spec()
-
-            spec = exp.WindowSpec(
-                kind=kind,
-                start=start['value'],
-                start_side=start['side'],
-                end=end['value'],
-                end_side=end['side'],
-            )
-
-        if not self._match(TokenType.R_PAREN):
-            self.raise_error('Expecting )')
-
-        return exp.Window(this=this, partition=partition, order=order, spec=spec)
+    def _parse_dot(self, this):
+        while self._match(TokenType.DOT):
+            this = exp.Dot(this=this, expression=self._parse_id_var())
+        return this
 
     def _parse_window_spec(self):
         self._match(TokenType.BETWEEN)
@@ -747,11 +782,9 @@ class Parser:
 
         return items
 
-    def _parse_tokens(self, parse, *expressions):
+    def _parse_tokens(self, parse, expressions):
         this = parse()
 
-        expressions = {expression.token_type: expression for expression in expressions}
-
         while self._match(*expressions):
             this = expressions[self._prev.token_type](this=this, expression=parse())
 
